﻿// MainWindow.fs
// Copyright 2016, OSTRA / Takeshi Yoneki

// UIとしてのウィンドウとコマンドの実体。
module MainWindow

open System
open System.Windows
open System.Windows.Input
open System.Windows.Controls
open System.ComponentModel
open System.Collections.ObjectModel
open ViewModels

// コントロール
let menuItemPlay (w: Window) =
    w.FindName("menuItemPlay") :?> MenuItem

let menuItemReload (w: Window) =
    w.FindName("menuItemReload") :?> MenuItem

let menuItemCopy (w: Window) =
    w.FindName("menuItemCopy") :?> MenuItem

let lvHits (w: Window) =
    w.FindName("lvHits") :?> ListView

let tbSearchText (w: Window) =
    w.FindName("tbSearchText") :?> TextBox

// モデルのインスタンス
let viewModel (w: Window) =
    w.DataContext :?> MainViewModel

// メニューの機能
let OnPlayCommand w (obj: Object) (arg: ExecutedRoutedEventArgs) =
    let lv = lvHits w
    if lv.SelectedItem <> null then
        let item = lv.SelectedItem :?> MpdDB.MusicItem
        item.Play()
    ()

let CanPlayCommand w (obj: Object) (arg: CanExecuteRoutedEventArgs) =
    arg.CanExecute <- true

let OnReloadCommand w (obj: Object) (arg: ExecutedRoutedEventArgs) =
    (viewModel w).SetRecordList(MpdDB.LoadDB ())

let CanReloadCommand w (obj: Object) (arg: CanExecuteRoutedEventArgs) =
    arg.CanExecute <- true

let OnCopyCommand w (obj: Object) (arg: ExecutedRoutedEventArgs) =
    let st = tbSearchText w
    let lv = lvHits w
    if st.IsFocused then
        st.Copy()
    elif  lv.SelectedItem <> null then
        let item = lv.SelectedItem :?> MpdDB.MusicItem
        Clipboard.Clear()
        Clipboard.SetText(item.GetTextForClip(), TextDataFormat.UnicodeText)

let CanCopyCommand w (obj: Object) (arg: CanExecuteRoutedEventArgs) =
    arg.CanExecute <- true

// イベントハンドラ
let lvHits_MouseDoubleClick w (arg: MouseButtonEventArgs) =
    let lv = lvHits w
    if lv.SelectedItem <> null then
        let elm = lv.ItemContainerGenerator.ContainerFromItem(lv.SelectedItem) :?> UIElement
        if elm.InputHitTest(arg.GetPosition(elm)) <> null then
            (viewModel w).PlayCommand.Execute(null, null)
    ()

let mainWindow_Closed w (arg: EventArgs) =
    // 状態の保存。
    let gv = (lvHits w).View :?> GridView
    if w.WindowState = WindowState.Normal then
        Prefs.data.Left <- w.Left
        Prefs.data.Top <- w.Top
        Prefs.data.Width <- w.Width
        Prefs.data.Height <- w.Height
    Prefs.data.Column0Width <- gv.Columns.[0].Width
    Prefs.data.Column1Width <- gv.Columns.[1].Width
    Prefs.data.Column2Width <- gv.Columns.[2].Width
    Prefs.data.Column3Width <- gv.Columns.[3].Width
    Prefs.data.Column4Width <- gv.Columns.[4].Width
    Prefs.data.Column5Width <- gv.Columns.[5].Width
    Prefs.data.Column6Width <- gv.Columns.[6].Width
    Prefs.SavePreferences ()

// コンストラクタ
let Create () =
    // XAMLをWindowとしてロードする。
    let w = Application.LoadComponent(new System.Uri("/MpdDBReader2;component/MainWindow.xaml", UriKind.Relative)) :?> Window;
    // モデルはXAMLでインスタンスが作られWindowのDataContextに収められる。
    // LoadComponent後にモデルのインスタンスをDataContextに入れた場合（プログラムでDataContextを設定した場合）、
    // メニューアイテムのCommandはnullになっている。

    // ハンドラやコマンドの実体の登録。
    (lvHits w).MouseDoubleClick.Add(lvHits_MouseDoubleClick w)
    w.Closed.Add(mainWindow_Closed w)
    let _ = w.CommandBindings.Add(CommandBinding((menuItemCopy w).Command, ExecutedRoutedEventHandler(OnCopyCommand w), CanExecuteRoutedEventHandler (CanCopyCommand w)))
    let _ = w.CommandBindings.Add(CommandBinding((menuItemPlay w).Command, ExecutedRoutedEventHandler(OnPlayCommand w), CanExecuteRoutedEventHandler (CanPlayCommand w)))
    let _ = w.CommandBindings.Add(CommandBinding((menuItemReload w).Command, ExecutedRoutedEventHandler(OnReloadCommand w), CanExecuteRoutedEventHandler (CanReloadCommand w)))

    // 保存した状態の反映。
    Prefs.LoadPreferences ()
    let gv = (lvHits w).View :?> GridView
    w.Left <- Prefs.data.Left
    w.Top <- Prefs.data.Top
    w.Width <- Prefs.data.Width
    w.Height <- Prefs.data.Height
    gv.Columns.[0].Width <- Prefs.data.Column0Width
    gv.Columns.[1].Width <- Prefs.data.Column1Width
    gv.Columns.[2].Width <- Prefs.data.Column2Width
    gv.Columns.[3].Width <- Prefs.data.Column3Width
    gv.Columns.[4].Width <- Prefs.data.Column4Width
    gv.Columns.[5].Width <- Prefs.data.Column5Width
    gv.Columns.[6].Width <- Prefs.data.Column6Width

    // DBの読み込み。
    (viewModel w).SetRecordList(MpdDB.LoadDB ())
    //
    w
