﻿// MpdDB.fs
// Copyright 2016, OSTRA / Takeshi Yoneki

module MpdDB

// mpdの楽曲データベースであるtag_cache
// 以前はテキストだったが現在はgzip圧縮されている。展開すれば同様のテキスト。
// icalやvcalに似た
// 要素名: 値
// の羅列。

open System
open System.IO
open System.Diagnostics
open System.IO.Compression

type MusicItem() =
    member val Artist = "" with get, set
    member val Title = "" with get, set
    member val Album = "" with get, set
    member val Time = "" with get, set
    member val Year = "" with get, set
    member val Genre = "" with get, set
    member val Path = "" with get, set
    member val Track = "" with get, set
    member this.Play() = 
        let path = Prefs.data.DirPath + "\\" + this.Path.Replace('/', '\\')
        if File.Exists(path) then
            let _ = Process.Start(path)
            ()
        ()
    member this.GetTextForClip() =
        this.Artist + "\t" + this.Title + "\t" + this.Album + "\t" + this.Year + "\n"

let CommonComparer (item1: MusicItem) (item2: MusicItem) = 
    if item1.Artist < item2.Artist then
        -1
    elif item1.Artist > item2.Artist then
        1
    elif item1.Year < item2.Year then
        -1
    elif item1.Year > item2.Year then
        1
    elif item1.Album < item2.Album then
        -1
    elif item1.Album > item2.Album then
        1
    elif item1.Track < item2.Track then
        -1
    elif item1.Track > item2.Track then
        1
    elif item1.Path < item2.Path then
        -1
    elif item1.Path > item2.Path then
        1
    elif item1.Title < item2.Title then
        -1
    elif item1.Title > item2.Title then
        1
    else
        0

let rec MakeTrackValue (v: String) =
    if v.Length >= 4 then
        v
    else
        MakeTrackValue ("0" + v)

// mpdがtag_cacheをgzip圧縮するようになった。
// 2016.08.07
let ReadAllLinesWithGzipDecompress path =
    use inStream = File.OpenRead(path)
    use gzStream = new GZipStream(inStream, CompressionMode.Decompress)
    use sr = new StreamReader(gzStream)
    let rec loop acc =
        match sr.ReadLine() with
        | null -> acc
        | line -> loop (line :: acc)
    List.rev (loop [])

let LoadDB () = 
    if File.Exists(Prefs.data.DBPath) then
        let lines = ReadAllLinesWithGzipDecompress(Prefs.data.DBPath)
        // gzip圧縮される以前のバージョンなら以下のFile.ReadAllLinesに変更する。
        //let lines = File.ReadAllLines(Pref.pref.DBPath)
        let (result, _, _) = 
            Seq.fold
                (fun (records, current: Option<MusicItem>, pathStack) (line: string) ->
                    let ss = line.Split([|':'|], 2)
                    let (a, b) =
                        match ss.Length with
                        | 0 -> ("", "")
                        | 1 -> (ss.[0], "")
                        | _ -> (ss.[0], ss.[1])
                    let (tag, value) = (a.Trim().ToLower(), b.Trim())
                    match tag with
                    | "begin" -> (records, current, value :: pathStack)
                    | "end" -> (records, current, List.tail pathStack)
                    | "song_begin" ->
                        let r = new MusicItem()
                        let path = List.head pathStack
                        r.Path <- 
                            if path.Length > 0 then
                                path + "/" + value
                            else
                                value
                        (records, Some r, pathStack)
                    | _ ->
                        if current <> None then
                            if tag = "song_end" then
                                (current.Value :: records, None, pathStack)
                            else 
                                match tag with
                                | "artist" -> current.Value.Artist <- value
                                | "title" -> current.Value.Title <- value
                                | "album" -> current.Value.Album <- value
                                | "date" -> current.Value.Year <- value
                                | "genre" -> current.Value.Genre <- value
                                | "track" -> current.Value.Track <- MakeTrackValue value
                                | "time" -> 
                                    let s = 
                                        try
                                            Convert.ToInt32(Double.Parse(value))
                                        with _ ->
                                            0
                                    current.Value.Time <- (s / 60).ToString() + ":" + (s % 60).ToString("00")
                                | _ -> ()
                                (records, current, pathStack)
                        else
                            (records, current, pathStack)
                )
                ([], None, [""])
                lines
        List.sortWith CommonComparer (Seq.toList result) 
    else
        []
