﻿// Prefs.fs
// Copyright 2016, OSTRA / Takeshi Yoneki

module Prefs

// C#のSettings.settingsのような仕組みはF#にはないので、昔ながらに自力でPreferencesを保存する。

open System
open System.Configuration
open System.Xml.Serialization
open System.IO

let settingDirPath = Path.Combine(System.Environment.GetFolderPath(Environment.SpecialFolder.ApplicationData), "OSTRA")
let settingFilePath = Path.Combine(settingDirPath, "MpdDBReader_2_0.pref")

type Preferences() =
    member val DBPath = "\\\\aleph\\home\\mpd\\audio\\tag_cache" with get, set
    member val DirPath = "\\\\aleph\\home\\audio" with get, set
    member val Left = 50.0 with get, set
    member val Top = 50.0 with get, set
    member val Width = 500.0 with get, set
    member val Height = 300.0 with get, set
    member val Column0Width = nan with get, set
    member val Column1Width = nan with get, set
    member val Column2Width = nan with get, set
    member val Column3Width = nan with get, set
    member val Column4Width = nan with get, set
    member val Column5Width = nan with get, set
    member val Column6Width = nan with get, set

let mutable data = new Preferences()

let LoadPreferences () =
    if File.Exists(settingFilePath) then
        use fs = new FileStream(settingFilePath, FileMode.Open)
        data <- (new XmlSerializer(typeof<Preferences>)).Deserialize(fs) :?> Preferences

let SavePreferences () =
    if not (File.Exists(settingDirPath)) then
        let _ = Directory.CreateDirectory(settingDirPath)
        ()
    use fs = new FileStream(settingFilePath, FileMode.Create)
    (new XmlSerializer(typeof<Preferences>)).Serialize(fs, data)
