﻿// Program.fs
// Copyright 2016, OSTRA / Takeshi Yoneki

module Program

// テンプレート「F# Empty Windows App (WPF)」を使わずにF#でWPFアプリケーションを作る。
// http://putridparrot.com/blog/creating-a-wpf-application-in-f/

open System
open System.Windows
open System.Threading
open System.Runtime.Remoting
open System.Runtime.Remoting.Channels
open System.Runtime.Remoting.Channels.Ipc

let APPL_NAME = "MpdDBReader2"
// 多重起動チェックのMutex
let appMutex = new Mutex(false, APPL_NAME)
// メインウィンドウ
let mutable mainWindow: Window = null

// プロセス間通信でのやり取りに使う型。
let REMOTECALL_NAME = "RemoteCall"
type RemoteCall() =
    inherit MarshalByRefObject()
    member this.Apply(argv) =
        if mainWindow <> null then
            // このスレッドはUIスレッドではないのでDispatcher.Invokeで呼び出す。
            mainWindow.Dispatcher.Invoke
                (fun () ->
                    if mainWindow.WindowState = WindowState.Minimized then
                        mainWindow.WindowState <- WindowState.Normal
                    // ウィンドウを（なるべく）前面にする。
                    let _ = mainWindow.Activate()
                    ()
                )

[<STAThread>]
[<EntryPoint>]
let main argv = 
    // Mutexを使って多重起動時の分岐をする。
    if appMutex.WaitOne(0, false) then
        // 最初の起動ならプロセス間通信のサーバを立てる。
        let serverChannel = new IpcServerChannel(APPL_NAME)
        ChannelServices.RegisterChannel(serverChannel, true)
        RemotingConfiguration.RegisterWellKnownServiceType(typeof<RemoteCall>, REMOTECALL_NAME, WellKnownObjectMode.Singleton)
        mainWindow <- MainWindow.Create ()
        (new Application()).Run(mainWindow)
    else
        // 二度目以降の起動ならプロセス間通信サーバにトリガーを渡して終了する。
        let clientChannel = new IpcClientChannel()
        ChannelServices.RegisterChannel(clientChannel, true)
        RemotingConfiguration.RegisterWellKnownClientType(typeof<RemoteCall>, "ipc://" + APPL_NAME + "/" + REMOTECALL_NAME)
        let remote = new RemoteCall()
        remote.Apply(argv)
        -1
